#include "Measurement2D.h"

Measurement2D::Measurement2D() { }

Measurement2D::~Measurement2D() { }

double Measurement2D::x() const { return measurement_x; }
double Measurement2D::y() const { return measurement_y; }

Measurement2D::Matrix Measurement2D::errorMatrix() const { 
  return measurement_error;
}

double Measurement2D::value() const { return sqrt(x()*x() + y()*y());}

double Measurement2D::error() const {
  // V' = JVJ^T (Jacobian*Errormatrix*JacobianTranspose)
  // Error propagation, see e.g Glen Cowan, Statistical Analysis
  
  // Errormatrix*JacobianTranspose
  double xx = errorMatrix().xx*x() + errorMatrix().xy*y();
  double yy = errorMatrix().yx*x() + errorMatrix().yy*y();
  
  // Jacobian*Errormatrix*JacobianTranspose
  double theError = ( x()*xx +y()*yy )/( x()*x() + y()*y() ); // division by r from the J=(x/r,y/r)
  return sqrt(theError);
}

double Measurement2D::significance() const {
  double significance = 0;
  if (error()>0) significance = value()/error();	
  return significance;
}


double Measurement2D::errorMethod2() const {
  // Alternative way to calculate the error
  // (scales nicely for larger dimensions)

  // Construct the data tables
  int dim = 2;
  double myR = value();
  double myDistance[2] = { measurement_x / myR, measurement_y / myR };
  double myMatrix[2][2];
  myMatrix[0][0] = measurement_error.xx;
  myMatrix[0][1] = measurement_error.xy;
  myMatrix[1][0] = measurement_error.yx;
  myMatrix[1][1] = measurement_error.yy;
  // Do the actual calculation
  double mySum = 0.0;
  for (int i = 0; i < dim; ++i)
    for (int j = 0; j < dim; ++j)
      mySum += myDistance[i]*myMatrix[i][j]*myDistance[j];
  return sqrt(mySum);
}

double Measurement2D::significanceMethod2() const {
  double significance = 0;
  double myError = errorMethod2();
  if (myError > 0) significance =  value() / myError;	
  return significance;
}

void Measurement2D::setXY(double x,double y){
  measurement_x = x;
  measurement_y = y;
}

void Measurement2D::setErrorMatrix(Matrix& m){
  measurement_error = m;
}
